# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"

import datetime
import numpy as np
import matplotlib.pyplot as plt

import geospacelab.visualization.mpl.geomap.geodashboards as geomap


def test_tec():

    dt_fr = datetime.datetime(2015, 2, 14, 18)
    dt_to = datetime.datetime(2015, 2, 14, 23)
    viewer = geomap.GeoDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (15, 10)})
    viewer.dock(datasource_contents=['madrigal', 'gnss', 'tecmap'])
    viewer.set_layout(2, 3, wspace=0.5)

    tec = viewer.assign_variable('TEC_MAP', dataset_index=1)
    dts = viewer.assign_variable('DATETIME', dataset_index=1).value.flatten()
    glat = viewer.assign_variable('GEO_LAT', dataset_index=1).value
    glon = viewer.assign_variable('GEO_LON', dataset_index=1).value

    time1 = datetime.datetime(2015, 2, 14, 18, 30)
    ind_t = np.where(dts == time1)[0]

    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='S', ut=time1, mirror_south=True)
    panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0, pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='S', ut=time1,
    #                          boundary_lat=0, mirror_south=False)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='N', ut=time1,
    #                        boundary_lat=30, mirror_south=False)
    panel1.overlay_coastlines()
    panel1.overlay_gridlines()
    #
    tec_ = tec.value[ind_t[0], :, :]
    pcolormesh_config = tec.visual.plot_config.pcolormesh
    pcolormesh_config.update(c_lim=[5, 25])

    import geospacelab.visualization.mpl.colormaps as cm
    pcolormesh_config.update(cmap='jet')
    ipc = panel1.overlay_pcolormesh(tec_, coords={'lat': glat, 'lon': glon, 'height': 250.}, cs='GEO', shading='auto', **pcolormesh_config)
    panel1.add_colorbar(ipc, c_label="TECU", c_scale='linear', left=1.1, bottom=0.1, width=0.05, height=0.7)
    #
    # # viewer.add_text(0.5, 1.1, "dashboard title")
    panel1().text(0.3, 1.1, time1.strftime("%Y-%m-%d %H:%M"), transform=panel1().transAxes)

    ####
    time1 = datetime.datetime(2015, 2, 14, 18, 50)
    ind_t = np.where(dts == time1)[0]

    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=1, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='S', ut=time1, mirror_south=True)
    panel1 = viewer.add_polar_map(row_ind=0, col_ind=1, style='lst-fixed', cs='GEO', lst_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0, pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='S', ut=time1,
    #                          boundary_lat=0, mirror_south=False)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='N', ut=time1,
    #                        boundary_lat=30, mirror_south=False)
    panel1.overlay_coastlines()
    panel1.overlay_gridlines()
    #
    tec_ = tec.value[ind_t[0], :, :]
    pcolormesh_config = tec.visual.plot_config.pcolormesh
    pcolormesh_config.update(c_lim=[5, 25])

    import geospacelab.visualization.mpl.colormaps as cm
    pcolormesh_config.update(cmap='jet')
    ipc = panel1.overlay_pcolormesh(tec_, coords={'lat': glat, 'lon': glon, 'height': 250.}, cs='GEO', **pcolormesh_config)
    panel1.add_colorbar(ipc, c_label="TECU", c_scale='linear', left=1.1, bottom=0.1, width=0.05, height=0.7)
    #
    # # viewer.add_text(0.5, 1.1, "dashboard title")
    panel1().text(0.3, 1.1, time1.strftime("%Y-%m-%d %H:%M"), transform=panel1().transAxes)

    ####
    time1 = datetime.datetime(2015, 2, 14, 19, 10)
    ind_t = np.where(dts == time1)[0]

    panel1 = viewer.add_polar_map(row_ind=0, col_ind=2, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0, pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='S', ut=time1,
    #                          boundary_lat=0, mirror_south=False)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='N', ut=time1,
    #                        boundary_lat=30, mirror_south=False)

    panel1.overlay_coastlines()
    panel1.overlay_gridlines()
    #
    tec_ = tec.value[ind_t[0], :, :]
    pcolormesh_config = tec.visual.plot_config.pcolormesh
    pcolormesh_config.update(c_lim=[5, 25])

    import geospacelab.visualization.mpl.colormaps as cm
    pcolormesh_config.update(cmap='jet')
    ipc = panel1.overlay_pcolormesh(tec_, coords={'lat': glat, 'lon': glon, 'height': 250.}, cs='GEO', **pcolormesh_config)
    panel1.add_colorbar(ipc, c_label="TECU", c_scale='linear', left=1.1, bottom=0.1, width=0.05, height=0.7)
    #
    # # viewer.add_text(0.5, 1.1, "dashboard title")
    panel1().text(0.3, 1.1, time1.strftime("%Y-%m-%d %H:%M"), transform=panel1().transAxes)

    ####
    time1 = datetime.datetime(2015, 2, 14, 19, 30)
    ind_t = np.where(dts == time1)[0]

    panel1 = viewer.add_polar_map(row_ind=1, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0, pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='S', ut=time1,
    #                          boundary_lat=0, mirror_south=False)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='N', ut=time1,
    #                        boundary_lat=30, mirror_south=False)
    panel1.overlay_coastlines()
    panel1.overlay_gridlines()
    #
    tec_ = tec.value[ind_t[0], :, :]
    pcolormesh_config = tec.visual.plot_config.pcolormesh
    pcolormesh_config.update(c_lim=[5, 25])

    import geospacelab.visualization.mpl.colormaps as cm
    pcolormesh_config.update(cmap='jet')
    ipc = panel1.overlay_pcolormesh(tec_, coords={'lat': glat, 'lon': glon, 'height': 250.}, cs='GEO', **pcolormesh_config)
    panel1.add_colorbar(ipc, c_label="TECU", c_scale='linear', left=1.1, bottom=0.1, width=0.05, height=0.7)
    #
    # # viewer.add_text(0.5, 1.1, "dashboard title")
    panel1().text(0.3, 1.1, time1.strftime("%Y-%m-%d %H:%M"), transform=panel1().transAxes)

    ####
    time1 = datetime.datetime(2015, 2, 14, 19, 50)
    ind_t = np.where(dts == time1)[0]

    panel1 = viewer.add_polar_map(row_ind=1, col_ind=1, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0, pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='S', ut=time1,
    #                          boundary_lat=0, mirror_south=False)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='N', ut=time1,
    #                        boundary_lat=30, mirror_south=False)
    panel1.overlay_coastlines()
    panel1.overlay_gridlines()
    #
    tec_ = tec.value[ind_t[0], :, :]
    pcolormesh_config = tec.visual.plot_config.pcolormesh
    pcolormesh_config.update(c_lim=[5, 25])

    import geospacelab.visualization.mpl.colormaps as cm
    pcolormesh_config.update(cmap='jet')
    ipc = panel1.overlay_pcolormesh(tec_, coords={'lat': glat, 'lon': glon, 'height': 250.}, cs='GEO', **pcolormesh_config)
    panel1.add_colorbar(ipc, c_label="TECU", c_scale='linear', left=1.1, bottom=0.1, width=0.05, height=0.7)
    #
    # # viewer.add_text(0.5, 1.1, "dashboard title")
    panel1().text(0.3, 1.1, time1.strftime("%Y-%m-%d %H:%M"), transform=panel1().transAxes)

    ####
    time1 = datetime.datetime(2015, 2, 14, 20, 10)
    ind_t = np.where(dts == time1)[0]

    panel1 = viewer.add_polar_map(row_ind=1, col_ind=2, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0., pole='N', ut=time1, boundary_lat=50)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lst-fixed', cs='GEO', lst_c=0, pole='S', ut=time1, mirror_south=True)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='S', ut=time1,
    #                          boundary_lat=0, mirror_south=False)
    # panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='lon-fixed', cs='GEO', lon_c=0., pole='N', ut=time1,
    #                        boundary_lat=30, mirror_south=False)
    panel1.overlay_coastlines()
    panel1.overlay_gridlines()
    #
    tec_ = tec.value[ind_t[0], :, :]
    pcolormesh_config = tec.visual.plot_config.pcolormesh
    pcolormesh_config.update(c_lim=[5, 25])

    import geospacelab.visualization.mpl.colormaps as cm
    pcolormesh_config.update(cmap='jet')
    ipc = panel1.overlay_pcolormesh(tec_, coords={'lat': glat, 'lon': glon, 'height': 250.}, cs='GEO', **pcolormesh_config)
    panel1.add_colorbar(ipc, c_label="TECU", c_scale='linear', left=1.1, bottom=0.1, width=0.05, height=0.7)
    #
    # # viewer.add_text(0.5, 1.1, "dashboard title")
    panel1().text(0.4, 1.1, time1.strftime("%Y-%m-%d %H:%M"), transform=panel1().transAxes)


    plt.savefig('example_tec_aacgm_fixed_mlt2', dpi=200)
    plt.show()


if __name__ == "__main__":
    test_tec()



# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"

import numpy as np
import cartopy.crs as ccrs
import cartopy.feature as cfeature
import matplotlib.pyplot as plt
import matplotlib.ticker as mticker
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER

import geospacelab.visualization.mpl.panels as panels
import geospacelab.toolbox.utilities.pybasic as basic


class GeoPanelBase(panels.PanelBase):

    def __init__(self, *args, proj_class=None, proj_config: dict = None, **kwargs):

        self.proj_class = proj_class
        proj_config = proj_config if type(proj_config) is dict or proj_config is None else self._raise_error(TypeError)
        self.projection = self.proj_class(**proj_config)
        kwargs.update(projection=self.projection)
        super().__init__(*args, **kwargs)

    def add_axes(self, *args, major=False, label=None, **kwargs):
        if major:
            kwargs.setdefault('projection', self.projection)
        ax = super().add_axes(*args, major=major, label=label, **kwargs)
        return ax

    def overlay_coastlines(self, *args, **kwargs):
        cl = self().coastlines(**kwargs)
        return cl

    def overlay_gridlines(self, *args, **kwargs):
        gl = self().gridlines(
            crs=ccrs.PlateCarree(), draw_labels=True,
            **kwargs
        )
        return gl

    def overlay_lands(self, *args, **kwargs):

        ll = self().add_feature(cfeature.LAND, **kwargs)
        return ll

    def add_title(self, x=0.5, y=1.1, title=None, **kwargs):
        kwargs.setdefault('ha', 'center')
        kwargs.setdefault('va', 'baseline')
        super().add_title(x=x, y=y, title=title, **kwargs)
    
    @property
    def proj_class(self):
        return self._proj_class

    @proj_class.setter
    def proj_class(self, proj):
        if proj is None:
            proj = 'Stereographic'
        if isinstance(proj, str):
            proj = getattr(ccrs, proj)
        elif issubclass(proj, ccrs.Projection):
            proj = proj
        else:
            raise TypeError

        self._proj_class = proj



import datetime
import matplotlib.pyplot as plt

import geospacelab.visualization.mpl.dashboards as dashboards


def test():
    dt_fr = datetime.datetime(2018, 5, 1, 0)
    dt_to = datetime.datetime(2018, 7, 1, 0)
    dashboard = dashboards.TSDashboard(dt_fr=dt_fr, dt_to=dt_to)
    ds1 = dashboard.dock(datasource_contents=['cdaweb', 'omni'])
    ds1.load_data()

    Bx = dashboard.assign_variable('B_x_GSM')
    By = dashboard.assign_variable('B_y_GSM')
    Bz = dashboard.assign_variable('B_z_GSM')

    n_p = dashboard.assign_variable('n_p')
    v_sw = dashboard.assign_variable('v_sw')
    p_dyn = dashboard.assign_variable('p_dyn')

    ds2 = dashboard.dock(datasource_contents=['wdc', 'asysym'])

    sym_h = dashboard.assign_variable('SYM_H')

    ds3 = dashboard.dock(datasource_contents=['wdc', 'ae'])
    ae = dashboard.assign_variable('AE')
    au = dashboard.assign_variable('AU')
    al = dashboard.assign_variable('AL')

    ds4 = dashboard.dock(datasource_contents=['gfz', 'kpap'])
    kp = dashboard.assign_variable('Kp')

    layout = [[Bz, By], [v_sw],]
    # layout = [[Bz, By], [v_sw], [n_p], [sym_h]]
    dashboard.set_layout(panel_layouts=layout, hspace=0.1)
    dashboard.draw()
    # dashboard.save_figure(file_name='example_omni_6', append_time=False)
    pass


if __name__ == "__main__":
    test()
    plt.show()



# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"

from geospacelab.visualization.mpl.ts_viewer import TimeSeriesViewer
from geospacelab.visualization.mpl.__base__ import plt, FigureBase


def mpl_viewer(figure_class=FigureBase, **kwargs):
    """
    Create a viewer based on **matplotlib**.

    :param figure_class: Optionally use a custom :class:`GeospaceLab Canvas <geospacelab.visualization.mpl._base.Canvas>
    instance.
    :type figure_class: subclass of :class:`GeospaceLab Canvas <geospacelab.visualization.mpl._base.Canvas>
    :param kwargs: Optional keyword arguments as same as in ``plt.figure``
    :return: The canvas instance
    """
    kwargs.setdefault('FigureClass', figure_class)
    fig = plt.figure(**kwargs)
    return fig





# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"


import datetime
import numpy as np

from geospacelab.cs._cs_base import SpaceCSBase, SphericalCoordinates, CartesianCoordinates, SpaceCartesianCS, SpaceSphericalCS
import geospacelab.toolbox.utilities.pylogging as mylog
import geospacelab.toolbox.utilities.pybasic as pybasic


class GEO(SpaceSphericalCS):
    def __init__(self, coords=None, ut=None, vector=None, **kwargs):
        
        kwargs.setdefault('new_coords', ['lat', 'lon', 'height'])
        super().__init__(name='GEO', coords=coords, vector=vector, ut=ut, **kwargs)

    def to_AACGM(self, append_mlt=False):
        cs_geoc = self.to_GEOC(kind='sph')
        cs_new = cs_geoc.to_AACGM(append_mlt=append_mlt)
        return cs_new

    def to_APEX(self, append_mlt=False):
        cs_geoc = self.to_GEOC(kind='sph')
        cs_new = cs_geoc.to_APEX(append_mlt=append_mlt)
        return cs_new

    def to_LENU(self, lat_0=None, lon_0=None, height_0=None, kind='car', **kwargs):
        """
        Convert GEO to local LENU
        :param lat_0: geographic latitude
        :param lon_0: geographic longitude
        :param height_0: altiude from the sea level
        :return: LENU object.
        """

        cs_0 = GEO(coords={'lat': lat_0, 'lon': lon_0, 'height': height_0})

        cs_geoc = self.to_GEOC(kind='car')
        cs_new = cs_geoc.to_LENU(lat_0=cs_0['lat'], lon_0=cs_0['lon'], height_0=cs_0['height'], kind=kind)

        return cs_new

    def to_GEOC(self, kind='sph', **kwargs):
        from geospacelab.cs import geopack
        height = self.coords.height
        if self['lat_unit'] == 'deg':
            factor = np.pi / 180.
        elif self['lat_unit'] == 'rad':
            factor = 1.
        else:
            raise AttributeError
        lat = self['lat'] * factor

        if np.isscalar(lat) and np.isscalar(height):
            r, theta = geopack.geodgeo(height, lat, 1)
        else:
            if np.isscalar(lat) and not np.isscalar(height):
                heights = height.flatten()
                lats = np.empty_like(heights)
                lats[:] = lat
                shape = height.shape
            elif not np.isscalar(lat) and np.isscalar(height):
                lats = lat.flatten()
                heights = np.empty_like(lats)
                heights[:] = height
                shape = lat.shape
            else:
                lats = lat.flatten()
                heights = height.flatten()
                shape = lat.shape

            rs = np.empty_like(lats)
            thetas = np.empty_like(lats)
            for ind, lat1 in enumerate(lats):
                height1 = heights[ind]
                rs[ind], thetas[ind] = geopack.geodgeo(height1, lat1, 1)
            r = np.reshape(rs, shape)
            theta = np.reshape(thetas, shape)

        r = r / self.coords.Re            # unit in Re
        phi = self['lon'] * factor

        coords = {'r': r, 'theta': theta, 'phi': phi}

        vector = self.vector

        cs_new = GEOCSpherical(coords=coords, ut=self.ut, vector=vector)

        if kind == 'car':
            cs_new = cs_new.to_cartesian()
        return cs_new


class GEOD(GEO):
    def __init__(self, coords=None, ut=None, vector=None, **kwargs):
        super().__init__(name='GEO', coords=coords, vector=vector, ut=ut, **kwargs)


class LENUCartesian(SpaceCartesianCS):
    def __init__(self, coords=None, lat_0=None, lon_0=None, height_0=None, ut=None, vector=None, **kwargs):
        kwargs.setdefault('new_coords', ['E', 'N', 'Up', 'x', 'y', 'z'])
        self.lat_0 = lat_0
        self.lon_0 = lon_0
        self.height_0 = height_0
        super().__init__(name='LENU', coords=coords, ut=ut, vector=vector, **kwargs)

        self.check_coords()

    def check_coords(self):
        if self.coords.E is not None and self.coords.x is None:
            self.convert_ENU_to_xyz()
        elif self.coords.E is None and self.coords.x is not None:
            self.convert_xyz_to_ENU()

    def convert_ENU_to_xyz(self):
        x = self.coords.E / self.coords.Re
        y = self.coords.N / self.coords.Re
        z = self.coords.Up / self.coords.Re
        self.coords.x = x
        self.coords.y = y
        self.coords.z = z

    def convert_xyz_to_ENU(self):
        E = self.coords.x * self.coords.Re
        N = self.coords.y * self.coords.Re
        Up = self.coords.z * self.coords.Re
        self.coords.E = E
        self.coords.N = N
        self.coords.Up = Up

    def to_GEOC(self, kind='sph', **kwargs):

        cs_0 = GEO(coords={'lat': self.lat_0, 'lon': self.lon_0, 'height': self.height_0})
        cs_0 = cs_0.to_GEOC(kind='sph')
        phi_0 = cs_0['phi']
        theta_0 = cs_0['theta']
        cs_0 = cs_0.to_cartesian()
        x_0 = cs_0['x']
        y_0 = cs_0['y']
        z_0 = cs_0['z']

        R_1 = np.array([
            [1,     0,                  0],
            [0,     np.cos(theta_0),    np.sin(theta_0)],
            [0,     -np.sin(theta_0),    np.cos(theta_0)]
        ])

        R_2 = np.array([
            [-np.sin(phi_0),    np.cos(phi_0),     0],
            [-np.cos(phi_0),     -np.sin(phi_0),     0],
            [0,                 0,                  1]
        ])

        x = self.coords.x
        y = self.coords.y
        z = self.coords.z
        shape_x = x.shape

        v = np.array([x.flatten(), y.flatten(), z.flatten()]).T
        v_0 = np.array([x_0, y_0, z_0])
        v_0 = np.tile(v_0, (x.size, 1))

        v_new = v @ R_1 @ R_2 + v_0

        x_new = np.reshape(v_new[:, 0], shape_x)
        y_new = np.reshape(v_new[:, 1], shape_x)
        z_new = np.reshape(v_new[:, 2], shape_x)

        coords = {'x': x_new, 'y': y_new, 'z': z_new}

        vector = self.vector
        if vector is not None:
            raise NotImplementedError

        cs_new = GEOCCartesian(coords=coords, vector=vector)

        if kind == 'sph':
            cs_new = cs_new.to_spherical()
        return cs_new

    def to_spherical(self, **kwargs):
        cs_new = super(LENUCartesian, self).to_spherical(lat_0=self.lat_0, lon_0=self.lon_0, height_0=self.height_0)
        return cs_new

    def to_GEO(self, **kwargs):
        cs_new = self.to_GEOC()
        cs_new = cs_new.to_GEO()
        return cs_new


class LENUSpherical(SpaceSphericalCS):
    def __init__(self, coords=None, lat_0=None, lon_0=None, height_0=None, ut=None, vector=None, **kwargs):
        kwargs.setdefault('new_coords', ['az', 'el', 'range', 'theta', 'phi', 'r'])

        self.lat_0 = lat_0
        self.lon_0 = lon_0
        self.height_0 = height_0
        super().__init__(name='LENU', coords=coords, ut=ut, vector=vector, **kwargs)

        self.check_coords()

    def check_coords(self):
        if self.coords.phi is None and self.coords.az is not None:
            self.convert_rangeazel_to_rphitheta()
        elif self.coords.phi is not None and self.coords.az is None:
            self.convert_rphitheta_to_rangeazel()

    def convert_rangeazel_to_rphitheta(self):
        if self.coords.az_unit == 'deg':
            factor = np.pi / 180.
        elif self.coords.az_unit == 'rad':
            factor = 1.
        else:
            raise NotImplemented
        phi = np.mod(np.pi/2 - self.coords.az * factor, 2*np.pi)
        theta = np.pi / 2 - self.coords.el * factor
        r = self.coords.range / self.coords.Re
        self.coords.phi = phi
        self.coords.theta = theta
        self.coords.r = r
        return theta, phi, r

    def convert_rphitheta_to_rangeazel(self):
        if self.coords.phi_unit == 'deg':
            factor = 1
        elif self.coords.az_unit == 'rad':
            factor = 180. / np.pi
        else:
            raise NotImplemented
        az = np.mod(90. - self.coords.phi*factor, 360)
        el = 90. - self.coords.theta * factor
        range = self.coords.r * self.coords.Re

        self.coords.az = az
        self.coords.el = el
        self.coords.range = range
        return az, el, range

    def to_cartesian(self, **kwargs):
        cs_new = super(LENUSpherical, self).to_cartesian(lat_0=self.lat_0, lon_0=self.lon_0, height_0=self.height_0)
        return cs_new

    def to_GEO(self, **kwargs):
        cs_new = self.to_cartesian()
        cs_new = cs_new.to_GEO()
        return cs_new

    def to_GEOC(self, kind='sph', **kwargs):
        cs_new = self.to_cartesian()
        cs_new = cs_new.to_GEOC(kind=kind)
        return cs_new


class GEOCSpherical(SpaceSphericalCS):
    def __init__(self, coords=None, ut=None, vector=None, **kwargs):
        kwargs.setdefault('new_coords', ['phi', 'theta', 'r', 'lat', 'lon', 'height'])
        super().__init__(name='GEOC', coords=coords, ut=ut, vector=vector, **kwargs)

        self.check_coords()

    def check_coords(self):
        if self.coords.lat is not None and self.coords.theta is None:
            self.convert_latlon_to_thetaphi()
        elif self.coords.lat is None and self.coords.theta is not None:
            self.convert_thetaphi_to_latlon()
        if self.coords.height is None and self.coords.r is not None:
            self.convert_r_to_height()
        elif self.coords.height is not None and self.coords.r is None:
            self.convert_height_to_r()

    def convert_latlon_to_thetaphi(self):
        if self.coords.lon_unit == 'rad':
            factor = 1.
        elif self.coords.lat_unit == 'deg':
            factor = np.pi / 180.
        else:
            raise NotImplementedError
        theta = np.pi/2 - self.coords.lat * factor
        phi = self.coords.lon * factor
        self['phi'] = phi
        self['theta'] = theta
        return theta, phi

    def convert_thetaphi_to_latlon(self):

        if self.coords.phi_unit == 'rad':
            factor = 180. / np.pi
        elif self.coords.phi_unit == 'deg':
            factor = 1.
        else:
            raise NotImplementedError
        lat = 90. - self.coords.theta * factor
        lon = self.coords.phi * factor
        self['lat'] = lat
        self['lon'] = lon
        return lat, lon

    def convert_height_to_r(self):
        r = (self.coords.height + self.coords.Re) / self.coords.Re
        self['r'] = r
        return r

    def convert_r_to_height(self):
        if self.coords.r_unit == 'Re':
            factor = self.coords.Re
        elif self.coords.r_unit == 'km':
            factor = 1.
        else:
            raise NotImplementedError
        height = self.coords.r * factor - self.coords.Re
        self['height'] = height
        return height

    def to_LENU(self, kind='sph', **kwargs):
        cs_new = self.to_cartesian()
        return cs_new.to_LENU(kind=kind, **kwargs)

    def to_GEO(self, **kwargs):
        from geospacelab.cs import geopack

        r = self.coords.r * self.coords.Re
        theta = self.coords.theta

        if np.isscalar(r) and np.isscalar(theta):
            h, lat = geopack.geodgeo(r, theta, 1)
        else:
            if np.isscalar(theta) and not np.isscalar(r):
                rs = r.flatten()
                thetas = np.empty_like(rs)
                thetas[:] = theta
                shape = r.shape
            elif not np.isscalar(theta) and np.isscalar(r):
                thetas = theta.flatten()
                rs = np.empty_like(thetas)
                rs[:] = r
                shape = thetas.shape
            else:
                thetas = theta.flatten()
                rs = r.flatten()
                shape = theta.shape

            hs = np.empty_like(thetas)
            lats = np.empty_like(thetas)
            for ind, r1 in enumerate(rs):
                theta1 = thetas[ind]
                hs[ind], lats[ind] = geopack.geodgeo(r1, theta1, -1)
            h = np.reshape(hs, shape)
            lat = np.reshape(lats, shape)
        lon = self.coords.phi

        factor = 180. / np.pi

        coords = {'lat': lat*factor, 'lon': lon*factor, 'height': h}
        vector = self.vector
        # if vi is not None:
        #    raise NotImplementedError

        cs_new = GEO(coords=coords, vector=vector, ut=self.ut)
        return cs_new

    def to_AACGM(self, append_mlt=False, **kwargs):
        import aacgmv2 as aacgm
        from geospacelab.cs._aacgm import AACGM
        method_code = 'G2A'

        ut_type = type(self.ut)
        if ut_type is list:
            uts = np.array(self.ut)
        elif ut_type is np.ndarray:
            uts = self.ut
        lat_shape = self.coords.lat.shape
        lon_shape = self.coords.lon.shape
        if issubclass(self.ut.__class__, datetime.datetime):

            lat, lon, r = aacgm.convert_latlon_arr(
                in_lat=self.coords.lat.flatten(), in_lon=self.coords.lon.flatten(), height=self.coords.height.flatten(),
                dtime=self.ut, method_code=method_code
            )
        else:
            if uts.shape[0] != self.coords.lat.shape[0]:
                mylog.StreamLogger.error("Datetimes must have the same length as cs!")
                return
            lat = np.empty_like(self.coords.lat)
            lon = np.empty_like(self.coords.lon)
            r = np.empty_like(self.coords.lat)
            for ind_dt, dt in enumerate(uts.flatten()):
                # print(ind_dt, dt, cs.lat[ind_dt, 0])
                lat[ind_dt], lon[ind_dt], r[ind_dt] = aacgm.convert_latlon_arr(in_lat=self.coords.lat[ind_dt],
                                                                               in_lon=self.coords.lon[ind_dt],
                                                                               height=self.coords.height[ind_dt],
                                                                               dtime=dt, method_code=method_code)
        cs_new = AACGM(coords={'lat': lat.reshape(lat_shape),
                               'lon': lon.reshape(lon_shape),
                               'r': r.reshape(lat_shape), 'r_unit': 'R_E'},
                       ut=self.ut)
        if append_mlt:
            lon = lon.flatten()
            if issubclass(self.ut.__class__, datetime.datetime):
                mlt = aacgm.convert_mlt(lon, self.ut)
            else:
                lon = cs_new['lon']
                mlt = np.empty_like(lon)
                for ind_dt, dt in enumerate(self.ut.flatten()):
                    mlt[ind_dt] = aacgm.convert_mlt(lon[ind_dt], dt)
            cs_new['mlt'] = mlt.reshape(lon_shape)
        return cs_new

    def to_APEX(self, append_mlt=False, **kwargs):
        import apexpy as apex
        from geospacelab.cs._apex import APEX

        cs_in = self

        ut_type = type(cs_in.ut)
        if ut_type is list:
            uts = np.array(cs_in.ut)
        elif ut_type is np.ndarray:
            uts = cs_in.ut

        mlt = None
        if issubclass(self.ut.__class__, datetime.datetime):
            apex_obj = apex.Apex(cs_in.ut)
            mlat, mlon = apex_obj.convert(
                cs_in.coords.lat, cs_in.coords.lon, 'geo', 'apex', height=cs_in.coords.height,
            )
            if append_mlt:
                mlt = apex_obj.mlon2mlt(mlon, cs_in.ut)
        else:
            if uts.shape[0] != cs_in.coords.lat.shape[0]:
                mylog.StreamLogger.error("Datetimes must have the same length as cs!")
                return

            mlat = np.empty_like(cs_in.coords.lat)
            mlon = np.empty_like(cs_in.coords.lat)
            mlt = np.empty_like(cs_in.coords.lat)
            for ind_dt, dt in enumerate(uts.flatten()):
                # print(ind_dt, dt, cs.lat[ind_dt, 0])
                apex_obj = apex.Apex(dt)
                mlat[ind_dt], mlon[ind_dt] = apex_obj.convert(
                    cs_in.coords.lat[ind_dt], cs_in.coords.lon[ind_dt], 'geo', 'apex',
                    height=cs_in.coords.height[ind_dt]
                )
                if append_mlt:
                    mlt[ind_dt] = apex_obj.mlon2mlt(mlon[ind_dt], dt)

        cs_new = APEX(coords={'lat': mlat, 'lon': mlon, 'height': cs_in.coords.height, 'mlt': mlt}, ut=cs_in.ut)

        return cs_new


class GEOCCartesian(SpaceCartesianCS):
    def __init__(self, coords=None, ut=None, vector=None, **kwargs):
        kwargs.setdefault('new_coords', ['x', 'y', 'z'])

        super().__init__(name='GEOC', coords=coords, ut=ut, vector=vector, **kwargs)

    def to_GEO(self, **kwargs):
        cs_new = self.to_spherical()
        return cs_new.to_GEO()

    def to_AACGM(self, append_mlt=False, **kwargs):
        cs_new = self.to_spherical()
        return cs_new.to_AACGM(append_mlt=append_mlt, **kwargs)

    def to_APEX(self, append_mlt=False, **kwargs):
        cs_new = self.to_spherical()
        return cs_new.to_APEX(append_mlt=append_mlt, **kwargs)

    def to_LENU(self, lat_0=None, lon_0=None, height_0=None, kind='sph'):
        cs_0 = GEOCSpherical(coords={'lat': lat_0, 'lon': lon_0, 'height': height_0})
        phi_0 = cs_0['phi']
        theta_0 = cs_0['theta']
        cs_0 = cs_0.to_cartesian()
        x_0 = cs_0['x']
        y_0 = cs_0['y']
        z_0 = cs_0['z']

        x = self.coords.x
        y = self.coords.y
        z = self.coords.z
        shape_x = x.shape

        v = np.array([x.flatten(), y.flatten(), z.flatten()]).T
        v_0 = np.array([x_0, y_0, z_0])
        v_0 = np.tile(v_0, (x.size, 1))

        R_1 = np.array([
            [-np.sin(phi_0), -np.cos(phi_0), 0],
            [np.cos(phi_0), -np.sin(phi_0), 0],
            [0, 0, 1]
        ])

        R_2 = np.array([
            [1, 0, 0],
            [0, np.cos(theta_0), -np.sin(theta_0)],
            [0, np.sin(theta_0), np.cos(theta_0)]
        ])

        v_new = (v - v_0) @ R_1 @ R_2

        x_new = np.reshape(v_new[:, 0], shape_x)
        y_new = np.reshape(v_new[:, 1], shape_x)
        z_new = np.reshape(v_new[:, 2], shape_x)
        cs_new = LENUCartesian(coords={'x': x_new, 'y': y_new, 'z': z_new}, lat_0=lat_0, lon_0=lon_0, kind='car')

        if kind == 'sph':
            cs_new = cs_new.to_spherical()
        return cs_new



# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"


import geospacelab.visualization.mpl as mpl
import geospacelab.datahub as datahub
from geospacelab.visualization.mpl.geomap.geopanels import PolarMapPanel

default_layout_config = {
    'left': 0.15,
    'right': 0.8,
    'bottom': 0.15,
    'top': 0.88,
    'hspace': 0.1,
    'wspace': 0.1
}

default_figure_config = {
    'figsize': (12, 12),    # (width, height)
    'dpi': 100,
}


class GeoDashboard(mpl.Dashboard):
    def __init__(self, **kwargs):
        figure = kwargs.pop('figure', 'new')
        figure_config = kwargs.pop('figure_config', default_figure_config)

        super().__init__(visual='on', figure_config=figure_config, figure=figure, **kwargs)

    def set_layout(self, num_rows=None, num_cols=None, left=None, right=None, bottom=None, top=None,
                   hspace=None, wspace=None, **kwargs):

        if left is None:
            left = default_layout_config['left']
        if right is None:
            right = default_layout_config['right']
        if bottom is None:
            bottom = default_layout_config['bottom']
        if top is None:
            top = default_layout_config['top']
        if hspace is None:
            hspace = default_layout_config['hspace']
        if hspace is None:
            wspace = default_layout_config['wspace']

        super().set_layout(num_rows=num_rows, num_cols=num_cols, left=left, right=right, bottom=bottom, top=top,
                           hspace=hspace, wspace=wspace, **kwargs)

    def add_polar_map(self, row_ind, col_ind, label=None, cs='GEO', style='lon-fixed', pole='N',
                      ut=None, lon_c=None, lst_c=None, mlt_c=None, mlon_c=None, boundary_lat=30.,
                      boundary_style='circle', mirror_south=False,
                      proj_type='Stereographic', **kwargs) -> PolarMapPanel:
        """

        :param pole:
        :param row_ind: the row
        :param style: 'lon-fixed', 'lst-fixed', 'mlt-fixed'  or 'mlon-fixed'
        :return:
        """

        panel = super().add_panel(row_ind=row_ind, col_ind=col_ind, panel_class=PolarMapPanel,
                                     label=label, cs=cs, style=style, pole=pole,
                                     ut=ut, lon_c=lon_c, lst_c=lst_c, mlt_c=mlt_c, mlon_c=mlon_c, boundary_lat=boundary_lat,
                                     boundary_style=boundary_style,
                                     mirror_south=mirror_south,
                                     proj_type=proj_type, **kwargs)
        return panel





import datetime
import numpy as np
import matplotlib.pyplot as plt

import geospacelab.visualization.mpl.geomap.geodashboards as geomap


def test_ssusi():
    dt_fr = datetime.datetime(2005, 9, 2, 8)
    dt_to = datetime.datetime(2005, 9, 2, 23, 59)
    time1 = datetime.datetime(2005, 9, 2, 10, 10)
    pole = 'N'
    sat_id = 'f16'
    band = 'LBHS'

    # Create a geodashboard object
    dashboard = geomap.GeoDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (5, 5)})

    # If the orbit_id is specified, only one file will be downloaded. This option saves the downloading time.
    # dashboard.dock(datasource_contents=['jhuapl', 'dmsp', 'ssusi', 'edraur'], pole='N', sat_id='f17', orbit_id='46863')
    # If not specified, the data during the whole day will be downloaded.
    ds_ssusi = dashboard.dock(datasource_contents=['jhuapl', 'dmsp', 'ssusi', 'sdrdisk'], pole=pole, sat_id=sat_id, orbit_id='09672', pp_type='DAY_AURORAL')
    ds_s1 = dashboard.dock(
        datasource_contents=['madrigal', 'satellites', 'dmsp', 's1'],
        dt_fr=time1 - datetime.timedelta(minutes=45),
        dt_to=time1 + datetime.timedelta(minutes=45),
        sat_id=sat_id, replace_orbit=True)

    dashboard.set_layout(1, 1)

    # Get the variables: LBHS emission intensiy, corresponding times and locations
    lbhs = dashboard.assign_variable('DISK_R_RECT_' + band, dataset_index=1)
    dts = dashboard.assign_variable('DATETIME', dataset_index=1)
    scdt = dashboard.assign_variable('SC_DATETIME', dataset_index=1)
    glat = dashboard.assign_variable('DISK_GEO_LAT', dataset_index=1)
    glon = dashboard.assign_variable('DISK_GEO_LON', dataset_index=1)
    alt = dashboard.assign_variable('DISK_GEO_ALT', dataset_index=1)
    # mlt = dashboard.assign_variable(('GRID_MLT'), dataset_index=1).value

    # Search the index for the time to plot, used as an input to the following polar map
    ind_t = dashboard.datasets[1].get_time_ind(ut=time1)
    if (dts.value[ind_t, 0] - time1).total_seconds()/60 > 60:     # in minutes
        raise ValueError("The time does not match any SSUSI data!")
    lbhs_ = lbhs.value[ind_t]
    lbhs_[lbhs_<1.] = 1.
    glat_ = glat.value[ind_t]
    glon_ = glon.value[ind_t]
    alt_ = alt.value[ind_t][0]
    
    glat_, glon_, lbhs_ = ds_ssusi.regriddata(disk_data = lbhs_, disk_geo_lat = glat_, disk_geo_lon= glon_)
    # Add a polar map panel to the dashboard. Currently the style is the fixed MLT at mlt_c=0. See the keywords below:
    panel1 = dashboard.add_polar_map(
        row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM',
        mlt_c=0., pole=pole, ut=time1, boundary_lat=55., mirror_south=True
    )

    # Some settings for plotting.
    pcolormesh_config = lbhs.visual.plot_config.pcolormesh
    scdt_ = scdt.value[ind_t][:, np.newaxis]
    # Overlay the SSUSI image in the map.
    ipc = panel1.overlay_pcolormesh(
        data=lbhs_, coords={'lat': glat_, 'lon': glon_, 'height': alt_}, ut=scdt_, cs='GEO', **pcolormesh_config, regridding=True, data_res=0.5, grid_res=0.05)
    # Add a color bar
    panel1.add_colorbar(ipc, c_label=band + " (R)", c_scale=pcolormesh_config['c_scale'], left=1.1, bottom=0.1,
                        width=0.05, height=0.7)

    # Overlay the gridlines
    panel1.overlay_gridlines(lat_res=5, lon_label_separator=5)

    # Overlay the coastlines in the AACGM coordinate
    panel1.overlay_coastlines()

    # Overlay cross-track velocity along satellite trajectory
    sc_dt = ds_s1['SC_DATETIME'].value.flatten()
    sc_lat = ds_s1['SC_GEO_LAT'].value.flatten()
    sc_lon = ds_s1['SC_GEO_LON'].value.flatten()
    sc_alt = ds_s1['SC_GEO_ALT'].value.flatten()
    sc_coords = {'lat': sc_lat, 'lon': sc_lon, 'height': sc_alt}

    v_H = ds_s1['v_i_H'].value.flatten()
    panel1.overlay_cross_track_vector(
        vector=v_H, unit_vector=1000, vector_unit='m/s', alpha=0.3, color='red',
        sc_coords=sc_coords, sc_ut=sc_dt, cs='GEO',
    )
    # Overlay the satellite trajectory with ticks
    panel1.overlay_sc_trajectory(sc_ut=sc_dt, sc_coords=sc_coords, cs='GEO')

    # Overlay sites
    panel1.overlay_sites(site_ids=['TRO', 'ESR'], coords={'lat': [69.58, 78.15], 'lon': [19.23, 16.02], 'height': 0.}, cs='GEO', marker='^', markersize=2)

    # Add the title and save the figure
    polestr = 'North' if pole == 'N' else 'South'
    panel1.add_title(title='DMSP/SSUSI, ' + band + ', ' + sat_id.upper() + ', ' + polestr + ', ' + time1.strftime('%Y-%m-%d %H%M UT'))
    plt.savefig('DMSP1_SSUSI_' + time1.strftime('%Y%m%d-%H%M') + '_' + band + '_' + sat_id.upper() + '_' + pole, dpi=300)

    # show the figure
    plt.show()


if __name__ == "__main__":
    test_ssusi()


import datetime
import matplotlib.pyplot as plt
import numpy as np

from geospacelab import preferences as pref
pref.user_config['visualization']['mpl']['style'] = 'dark'  # or 'light'

import geospacelab.visualization.mpl.dashboards as dashboards


def test_swarm_pod():
    # Set the starting and stopping time
    dt_fr = datetime.datetime(2015, 9, 8, 11)
    dt_to = datetime.datetime(2015, 9, 8, 18, 59)
    add_APEX = True     # if True, "SC_APEX_LAT" and "SC_APEX_LON" will be added, default is False

    # Create a dashboard object, equivalent to a datahub object, however, with the additional visulization control.
    db = dashboards.TSDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (12, 8)})

    # Dock the datasets. Different datasets store different types of data.
    # Dock the SWARM-A DNS-POD data
    ds_A = db.dock(datasource_contents=['tud', 'swarm', 'dns_pod'], sat_id='A', add_APEX=add_APEX)
    # Dock the SWARM-C DNS-POD data
    ds_C = db.dock(datasource_contents=['tud', 'swarm', 'dns_pod'], sat_id='C', add_APEX=add_APEX)

    # Assign variables from the datasets for visualization.
    rho_n_A = db.assign_variable('rho_n', dataset=ds_A)
    rho_n_C = db.assign_variable('rho_n', dataset=ds_C)
    rho_n_A.visual.axis[1].label = r'$\rho$'
    rho_n_A.visual.axis[2].label = 'Swarm-A'
    rho_n_C.visual.axis[2].label = 'Swarm-C'

    glat_A = db.assign_variable('SC_GEO_LAT', dataset=ds_A)
    glat_C = db.assign_variable('SC_GEO_LAT', dataset=ds_A)
    glat_A.visual.axis[2].label = 'Swarm-A'
    glat_C.visual.axis[2].label = 'Swarm-C'

    glon_A = db.assign_variable('SC_GEO_LON', dataset=ds_A)
    glon_C = db.assign_variable('SC_GEO_LON', dataset=ds_C)
    lst_A = db.assign_variable('SC_GEO_LST', dataset=ds_A)
    
    glon_A.visual.axis[2].label = 'Swarm-A'
    glon_C.visual.axis[2].label = 'Swarm-C'
    # Dock the dataset for the geomagnetic activity indices.
    ds1 = db.dock(datasource_contents=['wdc', 'asysym'])
    sym_h = db.assign_variable('SYM_H', dataset=ds1)

    # Set the plotting layout
    db.set_layout([[sym_h], [rho_n_A, rho_n_C], [glat_A, glat_C], [glon_A, [lst_A]]])
    db.draw()
    # plt.savefig('swarm_example', dpi=300)
    plt.show()

    # Extract the data array from variables:
    rho_n_A_array = rho_n_A.value


def test_swarm_pod_acc():
    # Set the starting and stopping time
    dt_fr = datetime.datetime(2015, 12, 21, 11)
    dt_to = datetime.datetime(2015, 12, 21, 18, 59)

    # Create a dashboard object, equivalent to a datahub object, however, with the additional visulization control.
    db = dashboards.TSDashboard(dt_fr=dt_fr, dt_to=dt_to, figure='new', figure_config={'figsize': (12, 8)})

    # Dock the datasets. Different datasets store different types of data.
    # Dock the SWARM-A DNS-POD data
    ds_pod = db.dock(datasource_contents=['tud', 'swarm', 'dns_pod'], sat_id='C')
    ds_acc = db.dock(datasource_contents=['tud', 'swarm', 'dns_acc'], sat_id='C')

    # Assign variables from the datasets for visualization.
    rho_n_pod = db.assign_variable('rho_n', dataset=ds_pod)
    rho_n_acc = db.assign_variable('rho_n', dataset=ds_acc)
    rho_n_pod.visual.axis[1].label = r'$\rho$'
    rho_n_pod.visual.axis[2].label = 'POD'
    rho_n_acc.visual.axis[2].label = 'ACC'

    glat = db.assign_variable('SC_GEO_LAT', dataset=ds_pod)
    glon = db.assign_variable('SC_GEO_LON', dataset=ds_pod)

    db.set_layout([[rho_n_pod, rho_n_acc], [glat], [glon]])
    db.draw()
    # plt.savefig('swarm_example', dpi=300)
    plt.show()


if __name__ == "__main__":
    test_swarm_pod()
    # test_swarm_pod_acc()


# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"

from geospacelab.datahub.sources.esa_eo.swarm.loader import LoaderModel

# define the default variable name dictionary
default_variable_name_dict = {
    'CDF_EPOCH': 'Timestamp',
    'SC_GEO_LAT': 'Latitude',
    'SC_GEO_LON': 'Longitude',
    'SC_GEO_ALT': 'Height',
    'SC_GEO_r': 'Radius',
    'SC_SZA': 'SZA',
    'SC_SAz': 'SAz',
    'SC_ST': 'ST',
    'SC_DIP_LAT': 'Diplat',
    'SC_DIP_LON': 'Diplon',
    'SC_QD_LAT': 'MLat',
    'SC_QD_MLT': 'MLT',
    'SC_AACGM_LAT': 'AACGMLat',
    'SC_AACGM_LON': 'AACGMLon',
    'n_e': 'n',
    'T_e_HGN': 'Te_hgn',
    'T_e_LGN': 'Te_lgn',
    'T_e': 'T_elec',
    'V_s_HGN': 'vs_hgn',
    'V_s_LGN': 'vs_lgn',
    # 'V_s': 'vs',
    'SC_U': 'U_SC',
    'QUALITY_FLAG': 'Flagbits'
}


class Loader(LoaderModel):
    """
    Load SWARM 2Hz or 16HZ TII data products. Currently support versions higher than "0301".

    The class is a hierarchy of :class:`SWARM data LoaderModel <geospacelab.datahub.sources.esa_eo.swarm.loader.LoaderModel>`

    """
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('variable_name_dict', default_variable_name_dict)
        super(Loader, self).__init__(*args, **kwargs)



import datetime
import matplotlib.pyplot as plt

import geospacelab.visualization.mpl.dashboards as dashboards


def test():
    dt_fr = datetime.datetime(2005, 8, 31, 12)
    dt_to = datetime.datetime(2005, 9, 2, 12)
    tsdb = dashboards.TSDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (15, 8)})
    ds1 = tsdb.dock(datasource_contents=['wdc', 'asysym'])
    ds2 = tsdb.dock(datasource_contents=['wdc', 'ae'])
    ds3 = tsdb.dock(datasource_contents=['wdc', 'dst'])
    
    ds4 = tsdb.dock(datasource_contents=['gfz', 'kpap'])
    ds5 = tsdb.dock(datasource_contents=['gfz', 'hpo'], data_res=60)
    # ds1.load_data()

    asy_d = tsdb.assign_variable('ASY_D', dataset=ds1)
    asy_h = tsdb.assign_variable('ASY_H', dataset=ds1)
    sym_d = tsdb.assign_variable('SYM_D', dataset=ds1)
    sym_h = tsdb.assign_variable('SYM_H', dataset=ds1)

    ae = tsdb.assign_variable('AE', dataset=ds2)
    au = tsdb.assign_variable('AU', dataset=ds2)
    al = tsdb.assign_variable('AL', dataset=ds2)
    ao = tsdb.assign_variable('AO', dataset=ds2)

    dst = tsdb.assign_variable('Dst', dataset=ds3)

    kp = tsdb.assign_variable('Kp', dataset=ds4)
    hp = tsdb.assign_variable('Hp', dataset=ds5)

    layout = [[dst], [asy_d, asy_h, sym_d, sym_h], [ae, au, ao, al], [kp], [hp]]
    tsdb.set_layout(panel_layouts=layout, hspace=0.1)
    tsdb.draw()
    plt.savefig('example_geomagnetic_indices.png')
    pass


if __name__ == "__main__":
    test()
    plt.show()



# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"

import numpy as np
import datetime

from geospacelab.datahub.__variable_base__ import VariableBase

def check_panel_ax(func):
    def wrapper(*args, **kwargs):
        obj = args[0]
        kwargs.setdefault('ax', None)
        if kwargs['ax'] is None:
            # kwargs['ax'] = obj.axes['major']
            kwargs['ax'] = obj.gca()
        result = func(*args, **kwargs)
        return result
    return wrapper





from tkinter import TRUE
import scipy.io as sio
import pathlib
import datetime
import numpy as np
import pandas as pd

from geospacelab.datahub import DatasetUser
from geospacelab.visualization.mpl.dashboards import TSDashboard
from geospacelab.cs import GEOCSpherical
import geospacelab.visualization.mpl.geomap.geodashboards as geomap


def visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N'):
    
    ds_swarm_pf = load_swarm_poynting_flux(swarm_dn, swarm_sat_id)
    
    band = 'LBHS'
    dt_fr = dmsp_dn - datetime.timedelta(minutes=60)
    dt_to = dmsp_dn + datetime.timedelta(minutes=60)
    time1 = dmsp_dn
    sat_id = dmsp_sat_id
    orbit_id = dmsp_orbit_id

    # Create a geodashboard object
    dashboard = geomap.GeoDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (18, 15)})

    # If the orbit_id is specified, only one file will be downloaded. This option saves the downloading time.
    # dashboard.dock(datasource_contents=['jhuapl', 'dmsp', 'ssusi', 'edraur'], pole='N', sat_id='f17', orbit_id='46863')
    # If not specified, the data during the whole day will be downloaded.
    dashboard.dock(datasource_contents=['jhuapl', 'dmsp', 'ssusi', 'edraur'], pole=pole, sat_id=sat_id, orbit_id=orbit_id)
    ds_s1 = dashboard.dock(
        datasource_contents=['madrigal', 'satellites', 'dmsp', 's1'],
        dt_fr=time1 - datetime.timedelta(minutes=45),
        dt_to=time1 + datetime.timedelta(minutes=45),
        sat_id=sat_id, replace_orbit=True)

    dashboard.set_layout(1, 1, left=0.05, right=0.3, top=0.7, hspace=0.1)

    # Get the variables: LBHS emission intensiy, corresponding times and locations
    lbhs = dashboard.assign_variable('GRID_AUR_' + band, dataset_index=1)
    dts = dashboard.assign_variable('DATETIME', dataset_index=1).value.flatten()
    mlat = dashboard.assign_variable('GRID_MLAT', dataset_index=1).value
    mlon = dashboard.assign_variable('GRID_MLON', dataset_index=1).value
    mlt = dashboard.assign_variable(('GRID_MLT'), dataset_index=1).value

    # Search the index for the time to plot, used as an input to the following polar map
    ind_t = dashboard.datasets[1].get_time_ind(ut=time1)
    if (dts[ind_t] - time1).total_seconds()/60 > 60:     # in minutes
        raise ValueError("The time does not match any SSUSI data!")
    lbhs_ = lbhs.value[ind_t]
    mlat_ = mlat[ind_t]
    mlon_ = mlon[ind_t]
    mlt_ = mlt[ind_t]
    # Add a polar map panel to the dashboard. Currently the style is the fixed MLT at mlt_c=0. See the keywords below:
    panel1 = dashboard.add_polar_map(
        row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM',
        mlt_c=0., pole=pole, ut=time1, boundary_lat=55., mirror_south=True
    )

    # Some settings for plotting.
    pcolormesh_config = lbhs.visual.plot_config.pcolormesh
    # Overlay the SSUSI image in the map.
    ipc = panel1.overlay_pcolormesh(
        data=lbhs_, coords={'lat': mlat_, 'lon': mlon_, 'mlt': mlt_}, cs='AACGM', **pcolormesh_config)
    # Add a color bar
    panel1.add_colorbar(ipc, c_label=band + " (R)", c_scale=pcolormesh_config['c_scale'], left=1.1, bottom=0.1,
                        width=0.05, height=0.7)

    # Overlay the gridlines
    panel1.overlay_gridlines(lat_res=5, lon_label_separator=5)

    # Overlay the coastlines in the AACGM coordinate
    panel1.overlay_coastlines()

    # Overlay cross-track velocity along satellite trajectory
    sc_dt = ds_s1['SC_DATETIME'].value.flatten()
    sc_lat = ds_s1['SC_GEO_LAT'].value.flatten()
    sc_lon = ds_s1['SC_GEO_LON'].value.flatten()
    sc_alt = ds_s1['SC_GEO_ALT'].value.flatten()
    sc_coords = {'lat': sc_lat, 'lon': sc_lon, 'height': sc_alt}

    v_H = ds_s1['v_i_H'].value.flatten()
    panel1.overlay_cross_track_vector(
        vector=v_H, unit_vector=1000, vector_unit='m/s', alpha=0.3, color='red',
        sc_coords=sc_coords, sc_ut=sc_dt, cs='GEO',
    )
    # Overlay the satellite trajectory with ticks
    panel1.overlay_sc_trajectory(sc_ut=sc_dt, sc_coords=sc_coords, cs='GEO')
    
    # Overlay swarm satellite trajectory
    sc2_dt = ds_swarm_pf['SC_DATETIME'].value.flatten()
    sc2_lat = ds_swarm_pf['SC_GEO_LAT'].value.flatten()
    sc2_lon = ds_swarm_pf['SC_GEO_LON'].value.flatten()
    sc2_alt = ds_swarm_pf['SC_GEO_ALT'].value.flatten()
    sc2_coords = {'lat': sc2_lat, 'lon': sc2_lon, 'height': sc2_alt}

    panel1.overlay_sc_trajectory(sc_ut=sc2_dt, sc_coords=sc2_coords, cs='GEO', color='m')

    # Overlay sites
    panel1.overlay_sites(site_ids=['TRO', 'ESR'], coords={'lat': [69.58, 78.15], 'lon': [19.23, 16.02], 'height': 0.}, cs='GEO', marker='^', markersize=2)

    # Add the title and save the figure
    polestr = 'North' if pole == 'N' else 'South'
    panel1.add_title(
            title='DMSP/SSUSI, ' + band + ', ' + sat_id.upper() + ', ' + polestr + ', ' + time1.strftime('%Y-%m-%d %H%M UT'))
    
    delta_t = 10
    dt_fr = swarm_dn - datetime.timedelta(minutes=delta_t)
    dt_to = swarm_dn + datetime.timedelta(minutes=delta_t)

    timeline_extra_labels = ['GEO_LAT', 'GEO_LON', 'AACGM_LAT', 'AACGM_MLT']
    db_swarm = TSDashboard(dt_fr=dt_fr, dt_to=dt_to, timeline_extra_labels=timeline_extra_labels, figure=dashboard.figure)
    
    db_swarm.add_dataset(ds_swarm_pf, kind='user-defined')
    
    pf_h = db_swarm.assign_variable('S_FA_H')
    pf_v = db_swarm.assign_variable('S_FA_V') 
    d_B_x = db_swarm.assign_variable('d_B_x')
    d_B_y = db_swarm.assign_variable('d_B_y')
    d_B_z = db_swarm.assign_variable('d_B_z')
    qflag = db_swarm.assign_variable('Q_FLAG')
    cflag = db_swarm.assign_variable('CALIB_FLAG')
    
    
    ds_lp = db_swarm.dock(
        datasource_contents=['esa_eo', 'swarm', 'advanced', 'efi_lp_hm'], 
        product='LP_HM', sat_id=swarm_sat_id, quality_control=False
        )

    n_e = db_swarm.assign_variable('n_e', dataset=ds_lp)
    T_e = db_swarm.assign_variable('T_e', dataset=ds_lp)
    
    ds_tii = db_swarm.dock(
        datasource_contents=['esa_eo', 'swarm', 'advanced', 'efi_tct02'], 
        product='TCT02', sat_id=swarm_sat_id, quality_control=False
        )

    v_i_H_x = db_swarm.assign_variable('v_i_H_x', dataset=ds_tii)
    v_i_H_y = db_swarm.assign_variable('v_i_H_y', dataset=ds_tii)
    v_i_V_x = db_swarm.assign_variable('v_i_V_x', dataset=ds_tii)
    v_i_V_z = db_swarm.assign_variable('v_i_V_z', dataset=ds_tii)
     
    db_swarm.set_layout([[n_e], [T_e], [d_B_x, d_B_y, d_B_z], [v_i_H_x, v_i_H_y, v_i_V_x, v_i_V_z], [pf_h, pf_v]],
                       left=0.5, right=0.9, top=0.7, hspace=0.6 )
    db_swarm.draw()
    
    """ SWARM/POD
    """
    try:
        db_rou = TSDashboard(dt_fr=dt_fr, dt_to=dt_to, figure=dashboard.figure)

        # Dock the datasets. Different datasets store different types of data.
        # Dock the SWARM-A DNS-POD data
        ds_pod = db_rou.dock(datasource_contents=['tud', 'swarm', 'dns_pod'], sat_id='C')
        ds_acc = db_rou.dock(datasource_contents=['tud', 'swarm', 'dns_acc'], sat_id='C')

        # Assign variables from the datasets for visualization.
        rho_n_pod = db_rou.assign_variable('rho_n', dataset=ds_pod)
        rho_n_acc = db_rou.assign_variable('rho_n', dataset=ds_acc)
        rho_n_pod.visual.axis[1].label = r'$\rho$'
        rho_n_pod.visual.axis[2].label = 'POD'
        rho_n_acc.visual.axis[2].label = 'ACC'

        glat = db_rou.assign_variable('SC_GEO_LAT', dataset=ds_pod)
        glon = db_rou.assign_variable('SC_GEO_LON', dataset=ds_pod)

        db_rou.set_layout([[rho_n_pod, rho_n_acc]], left=0.5, right=0.9, bottom=0.8, hspace=0.01)
        db_rou.draw()
    except:
        print(swarm_dn.strftime('%Y-%m-%d %H:%M UT'))
    
    db_swarm.add_title(title='SWARM-' + swarm_sat_id.upper() + ', ' + swarm_dn.strftime('%Y-%m-%d %H:%M UT'))
    db_swarm.save_figure(file_name = 'compare_v2_E' + swarm_dn.strftime('%Y-%m-%d') + '_SWARM-' + swarm_sat_id + '_DMSP-' + dmsp_sat_id.upper(), 
                         file_dir = pathlib.Path('/home/lei/01-Work/01-Project/OY22-IonosphereElectrodynamics/Lei_20220707/results'))
    # db_swarm.show()
    
    # return dashboard
    

def load_swarm_poynting_flux(dn0: datetime.datetime, sat_id):
    file_dir = pathlib.Path('/home/lei/01-Work/01-Project/OY22-IonosphereElectrodynamics/Lei_20220707/results')
    dstr = dn0.strftime('%Y%m%d-%H%M%S')
    file_path = list(file_dir.glob("*" + sat_id.upper() + "*" + dstr + '*.mat'))[0]
    matdata = sio.loadmat(file_path)
     
    ds = DatasetUser(visual='on')
    
    depend_0 = {
        'UT': 'SC_DATETIME', 
        'GEO_LAT': 'SC_GEO_LAT', 'GEO_LON': 'SC_GEO_LON', 
        'AACGM_LAT': 'SC_AACGM_LAT', 'AACGM_LON': 'SC_AACGM_LON', 'AACGM_MLT': 'SC_AACGM_MLT'
        }
    
    var_name = 'SC_DATETIME'
    var_value: np.ndarray = matdata['tl']
    ntl = var_value.shape[0]
    var_value = pd.to_datetime(var_value.flatten() - 719529, unit='D').to_numpy()
    var_value = [datetime.datetime.utcfromtimestamp(((var_value[i] - np.datetime64('1970-01-01T00:00:00')) / np.timedelta64(1, 's')))
                 for i in range(ntl)]
    var_value = np.array(var_value, dtype=datetime.datetime).reshape((ntl, 1))
    ut = var_value.flatten()
    var = ds.add_variable(var_name, value=var_value)
    var.visual.plot_config.style = '1P'
    
    var_name = 'SC_GEO_LAT'
    var_value: np.ndarray = matdata['glat']
    glat = var_value.flatten()
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.visual.plot_config.style = '1P'
    
    var_name = 'SC_GEO_LON'
    var_value: np.ndarray = matdata['glon']
    var_value = var_value.reshape((ntl, 1))
    glon = var_value.flatten()
    var = ds.add_variable(var_name, value=var_value)
    var.visual.plot_config.style = '1P'
    
    var_name = 'SC_GEO_R'
    var_value: np.ndarray = matdata['gR']
    var_value = var_value.reshape((ntl, 1))
    r = var_value.flatten()
    var = ds.add_variable(var_name, value=var_value)
    var.visual.plot_config.style = '1P'
    
    var_name = 'S_FA_V'
    var_value: np.np.ndarray = matdata['Pvpara']
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.depends[0] = depend_0
    var.visual.plot_config.style = '1P'
    var.visual.axis[1].label = 'S'
    var.visual.axis[1].unit = r'W$\cdot$m$^{-3}$'
    var.visual.axis[2].label = r'S$^V$'
    
    var_name = 'S_FA_H'
    var_value: np.ndarray = matdata['Phpara']
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.depends[0] = depend_0
    var.visual.plot_config.style = '1P'
    var.visual.axis[1].label = 'S'
    var.visual.axis[1].unit = r'W$\cdot$m$^{-3}$'
    var.visual.axis[2].label = r'S$^H$'    
    
    var_name = 'd_B_x'
    var_value: np.ndarray = matdata['Bx']
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.depends[0] = depend_0
    var.visual.plot_config.style = '1P'
    var.visual.axis[1].label = 'B'
    var.visual.axis[1].unit = 'nT'
    var.visual.axis[2].label = r'$\delta B_x$'
    
    var_name = 'd_B_y'
    var_value: np.ndarray = matdata['By']
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.depends[0] = depend_0
    var.visual.plot_config.style = '1P'
    var.visual.axis[2].label = r'$\delta B_y$'
    
    var_name = 'd_B_z'
    var_value: np.ndarray = matdata['Bz']
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.depends[0] = depend_0
    var.visual.plot_config.style = '1P'
    var.visual.axis[2].label = r'$\delta B_z$'
    
    var_name = 'Q_FLAG'
    var_value: np.ndarray = matdata['tmpQ']
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.depends[0] = depend_0
    var.visual.plot_config.style = '1P'
    var.visual.axis[1].label = 'FLAG'
    var.visual.axis[2].label = r'Quality=1 OK'
    
    var_name = 'CALIB_FLAG'
    var_value: np.ndarray = matdata['tmpC']
    var_value = var_value.reshape((ntl, 1))
    var = ds.add_variable(var_name, value=var_value)
    var.depends[0] = depend_0
    var.visual.plot_config.style = '1P'
    var.visual.axis[1].label = 'FLAG'
    var.visual.axis[2].label = r'Calib=0 OK'

    cs = GEOCSpherical(coords={'lat': glat, 'lon': glon, 'r': r/6371.2}, ut=ut)
    cs_new = cs.to_AACGM(append_mlt=True)
    var = ds.add_variable('SC_AACGM_LAT', value=cs_new['lat'])
    var = ds.add_variable('SC_AACGM_LON', value=cs_new['lon'])
    var = ds.add_variable('SC_AACGM_MLT', value=cs_new['mlt'])
    var = ds.add_variable('SC_GEO_ALT', value=ds['SC_GEO_R'].value - 6371.2)

    
    return ds


def event_1_1():
    dmsp_dn = datetime.datetime.strptime('20151221' + '131300', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f19'
    dmsp_orbit_id = '08855'
    
    swarm_dn = datetime.datetime.strptime('20151221' + '130400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='S')
    
    
def event_1_2():
    dmsp_dn = datetime.datetime.strptime('20151221' + '115700', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '62817'
    
    swarm_dn = datetime.datetime.strptime('20151221' + '130400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='S')
    
    
def event_1_3():
    dmsp_dn = datetime.datetime.strptime('20151221' + '145300', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f19'
    dmsp_orbit_id = '08856'
    
    swarm_dn = datetime.datetime.strptime('20151221' + '143600', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='S')    
    
def event_1_4():
    dmsp_dn = datetime.datetime.strptime('20151221' + '163300', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f19'
    dmsp_orbit_id = '08857'
    
    swarm_dn = datetime.datetime.strptime('20151221' + '160700', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='S')   
    
def event_1_5():
    dmsp_dn = datetime.datetime.strptime('20151221' + '133700', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '62818'
    
    swarm_dn = datetime.datetime.strptime('20151221' + '141000', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='S')       
    
def event_1_6():
    dmsp_dn = datetime.datetime.strptime('20151221' + '145300', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f19'
    dmsp_orbit_id = '08856'
    
    swarm_dn = datetime.datetime.strptime('20151221' + '141000', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='S') 
     
def event_1_7():
    dmsp_dn = datetime.datetime.strptime('20151221' + '151700', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '62819'
    
    swarm_dn = datetime.datetime.strptime('20151221' + '154400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='S')    
    
def event_2_1():
    dmsp_dn = datetime.datetime.strptime('20150908' + '183800', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f17'
    dmsp_orbit_id = '45631'
    
    swarm_dn = datetime.datetime.strptime('20150908' + '175800', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')   
    
def event_2_2():
    dmsp_dn = datetime.datetime.strptime('20150908' + '185900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '30373'
    
    swarm_dn = datetime.datetime.strptime('20150908' + '193100', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')  
            
def event_2_3():
    dmsp_dn = datetime.datetime.strptime('20150908' + '204000', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '30374'
    
    swarm_dn = datetime.datetime.strptime('20150908' + '210400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')  
    
def event_2_4():
    dmsp_dn = datetime.datetime.strptime('20150908' + '222100', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '30375'
    
    swarm_dn = datetime.datetime.strptime('20150908' + '223800', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')  

def event_2_5():
    dmsp_dn = datetime.datetime.strptime('20150908' + '171800', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '30372'
    
    swarm_dn = datetime.datetime.strptime('20150908' + '173600', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')   

def event_2_6():
    dmsp_dn = datetime.datetime.strptime('20150908' + '185900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '30373'
    
    swarm_dn = datetime.datetime.strptime('20150908' + '191100', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')   
    

def event_3_1():
    dmsp_dn = datetime.datetime.strptime('20140828' + '065600', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '25056'
    
    swarm_dn = datetime.datetime.strptime('20140828' + '072600', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 

def event_3_2():
    dmsp_dn = datetime.datetime.strptime('20140828' + '074700', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '56033'
    
    swarm_dn = datetime.datetime.strptime('20140828' + '072600', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 

def event_3_3():
    dmsp_dn = datetime.datetime.strptime('20140828' + '083900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '25057'
    
    swarm_dn = datetime.datetime.strptime('20140828' + '090000', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_3_4():
    dmsp_dn = datetime.datetime.strptime('20140828' + '092900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '56034'
    
    swarm_dn = datetime.datetime.strptime('20140828' + '090000', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')     
    
def event_3_5():
    dmsp_dn = datetime.datetime.strptime('20140828' + '065600', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '25056'
    
    swarm_dn = datetime.datetime.strptime('20140828' + '070300', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
        
def event_3_6():
    dmsp_dn = datetime.datetime.strptime('20140828' + '083900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '25057'
    
    swarm_dn = datetime.datetime.strptime('20140828' + '083600', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_4_1():
    dmsp_dn = datetime.datetime.strptime('20160803' + '024900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f17'
    dmsp_orbit_id = '50285'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '025100', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_4_2():
    dmsp_dn = datetime.datetime.strptime('20160803' + '024900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '35024'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '025100', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_4_3():
    dmsp_dn = datetime.datetime.strptime('20160803' + '043100', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f17'
    dmsp_orbit_id = '50286'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '042500', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_4_4():
    dmsp_dn = datetime.datetime.strptime('20160803' + '043100', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '35025'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '042500', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_4_5():
    dmsp_dn = datetime.datetime.strptime('20160803' + '061400', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '35026'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '060000', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 

def event_4_6():
    dmsp_dn = datetime.datetime.strptime('20160803' + '075700', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '35027'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '073400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 

def event_4_7():
    dmsp_dn = datetime.datetime.strptime('20160803' + '024900', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '35024'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '023300', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 

def event_4_8():
    dmsp_dn = datetime.datetime.strptime('20160803' + '034300', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '66006'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '040500', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_4_9():
    dmsp_dn = datetime.datetime.strptime('20160803' + '052600', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '66007'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '053900', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_4_10():
    dmsp_dn = datetime.datetime.strptime('20160803' + '070800', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f16'
    dmsp_orbit_id = '66008'
    
    swarm_dn = datetime.datetime.strptime('20160803' + '071400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N') 
    
def event_5_1():
    dmsp_dn = datetime.datetime.strptime('20170529' + '090400', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f17'
    dmsp_orbit_id = '54514'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '085100', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')     
    
def event_5_2():
    dmsp_dn = datetime.datetime.strptime('20170529' + '100200', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '39251'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '102400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')      
    
       
def event_5_3():
    dmsp_dn = datetime.datetime.strptime('20170529' + '104600', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f17'
    dmsp_orbit_id = '54515'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '102400', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')   
    
def event_5_4():
    dmsp_dn = datetime.datetime.strptime('20170529' + '114400', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '39252'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '115800', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')   
    
def event_5_5():
    dmsp_dn = datetime.datetime.strptime('20170529' + '132600', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '39253'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '133200', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')  
    
def event_5_6():
    dmsp_dn = datetime.datetime.strptime('20170529' + '150700', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '39254'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '150300', '%Y%m%d%H%M%S')
    swarm_sat_id = 'A'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')      
    
def event_5_7():
    dmsp_dn = datetime.datetime.strptime('20170529' + '114400', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '39252'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '114300', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')    
    
def event_5_8():
    dmsp_dn = datetime.datetime.strptime('20170529' + '132600', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '39253'
    
    swarm_dn = datetime.datetime.strptime('20170529' + '132000', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')        
    
def event_6_1():
    dmsp_dn = datetime.datetime.strptime('20170804' + '093600', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f18'
    dmsp_orbit_id = '40197'
    
    swarm_dn = datetime.datetime.strptime('20170804' + '093200', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')    

def event_6_2():
    dmsp_dn = datetime.datetime.strptime('20170804' + '092200', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f17'
    dmsp_orbit_id = '55461'
    
    swarm_dn = datetime.datetime.strptime('20170804' + '093200', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')    
    
def event_6_3():
    dmsp_dn = datetime.datetime.strptime('20170804' + '110400', '%Y%m%d%H%M%S')
    dmsp_sat_id = 'f17'
    dmsp_orbit_id = '55462'
    
    swarm_dn = datetime.datetime.strptime('20170804' + '110600', '%Y%m%d%H%M%S')
    swarm_sat_id = 'B'   
    
    visual_dmsp_swarm(dmsp_dn, dmsp_sat_id, dmsp_orbit_id, swarm_dn, swarm_sat_id, pole='N')        

if __name__ == '__main__':
    # event_1_1()
    # event_1_2()
    # event_1_3()
    # event_1_4()
    # event_1_5()
    # event_1_6()
    # event_1_7()
    
    # event_2_1()
    # event_2_2()
    # event_2_3()
    # event_2_4()
    # event_2_5()
    # event_2_6()
    
    # event_3_1()
    # event_3_2()
    # event_3_3()
    event_3_4()
    event_3_5()
    # event_3_6()
    
    # event_4_1()
    # event_4_2() 
    # event_4_3() 
    # event_4_4() 
    # event_4_5() 
    # event_4_6() 
    # event_4_7() 
    # event_4_8() 
    # event_4_9()
    # event_4_10()
      
    # event_5_1()
    # event_5_2() 
    # event_5_3() 
    # event_5_4()
    # event_5_5()
    # event_5_6()
    # event_5_7()
    # event_5_8()
    
    # event_6_1()
    # event_6_2() 
    # event_6_3() 


# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"


from geospacelab import preferences
import geospacelab.datahub.sources.madrigal.utilities as utilities

from geospacelab.datahub import DatabaseModel


class SuperMAGDatabase(DatabaseModel):
    def __new__(cls, str_in, **kwargs):
        obj = super().__new__(cls, str_in, **kwargs)
        return obj


supermag_database = SuperMAGDatabase('Madrigal')
supermag_database.url = 'https://supermag.jhuapl.edu/'
supermag_database.category = 'online database'


try:
    default_user_fullname = preferences.user_config['datahub']['supermag']['username']
    # default_user_email = preferences.user_config['datahub']['madrigal']['user_email']
    # default_user_affiliation = preferences.user_config['datahub']['madrigal']['user_email']
except KeyError:
    print("Inputs for accessing the SuperMAG database.")
    if preferences._on_rtd:
        default_username = 'GeospaceLab'
        # default_user_email = 'geospacelab@gmail.com'
        # default_user_affiliation = 'GeospaceLab'
        save = 'y'
    else:
        default_username = input("Username: ")
        # default_user_email = input("User's email: ")
        # default_user_affiliation = input("User's affiliation: ")
        save = input("Save as default? [y]/n: ")
    if save.lower() in ['', 'y', 'yes']:
        uc = preferences.user_config
        uc.setdefault('datahub', {})
        uc['datahub'].setdefault('madrigal', {})
        uc['datahub']['madrigal']['user_fullname'] = default_username
        # uc['datahub']['madrigal']['user_email'] = default_user_email
        # uc['datahub']['madrigal']['user_affiliation'] = default_user_affiliation
        preferences.set_user_config(user_config=uc, set_as_default=True)




import datetime
import pathlib
import re
import matplotlib.pyplot as plt

import geospacelab.visualization.mpl.geomap.geodashboards as geomap
from geospacelab.visualization.mpl.dashboards import TSDashboard
import geospacelab.toolbox.utilities.pydatetime as dttool
from geospacelab.observatory.sc_orbit import OrbitPosition_SSCWS


def show_dmsp(sat_id=None, pole='N', band='LBHS', dt_c=None, file_dir=None, orbit_id=None):
    file_name = 'DMSP_' + dt_c.strftime(
        '%Y-%m-%d_%H%M') + '_' + sat_id.upper() + '_SSUSI_' + band + '_SSJ_SSM_SSIES_' + pole + '.png'
    file_path_out: pathlib.Path = file_dir / file_name

    # if file_path_out.is_file():
    #     return
    dt_fr = dttool.get_start_of_the_day(dt_c)
    dt_to = dttool.get_end_of_the_day(dt_c)

    # Create a geodashboard object
    dashboard = geomap.GeoDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (16, 10)})

    # If the orbit_id is specified, only one file will be downloaded. This option saves the downloading time.
    # dashboard.dock(datasource_contents=['jhuapl', 'dmsp', 'ssusi', 'edraur'], pole='N', sat_id='f17', orbit_id='46863')
    # If not specified, the data during the whole day will be downloaded.

    dashboard.dock(datasource_contents=['jhuapl', 'dmsp', 'ssusi', 'edraur'], pole=pole, sat_id=sat_id, orbit_id=orbit_id,
                   time_clip=False)
    ds_s1 = dashboard.dock(
        datasource_contents=['madrigal', 'satellites', 'dmsp', 's1'],
        dt_fr=dt_c - datetime.timedelta(minutes=45),
        dt_to=dt_c + datetime.timedelta(minutes=45),
        sat_id=sat_id, replace_orbit=True)

    dashboard.set_layout(1, 1, left=0.05, right=0.35)

    try:
        # Get the variables: LBHS emission intensiy, corresponding times and locations
        lbhs = dashboard.assign_variable('GRID_AUR_' + band, dataset_index=1)
        dts = dashboard.assign_variable('DATETIME', dataset_index=1).value.flatten()
        mlat = dashboard.assign_variable('GRID_MLAT', dataset_index=1).value
        mlon = dashboard.assign_variable('GRID_MLON', dataset_index=1).value
        mlt = dashboard.assign_variable(('GRID_MLT'), dataset_index=1).value

        # Search the index for the time to plot, used as an input to the following polar map
        ind_t = dashboard.datasets[1].get_time_ind(ut=dt_c)
        if (dts[ind_t] - dt_c).total_seconds() / 60 > 60:  # in minutes
            raise ValueError("The time does not match any SSUSI data!")
        lbhs_ = lbhs.value[ind_t]
        mlat_ = mlat[ind_t]
        mlon_ = mlon[ind_t]
        mlt_ = mlt[ind_t]
        # Add a polar map panel to the dashboard. Currently the style is the fixed MLT at mlt_c=0. See the keywords below:
        panel1 = dashboard.add_polar_map(
            row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM',
            mlt_c=0., pole=pole, ut=dt_c, boundary_lat=55., mirror_south=True
        )

        # Some settings for plotting.
        pcolormesh_config = lbhs.visual.plot_config.pcolormesh
        pcolormesh_config['c_lim'] = [100, 2000]
        # Overlay the SSUSI image in the map.
        ipc = panel1.overlay_pcolormesh(
            data=lbhs_, coords={'lat': mlat_, 'lon': mlon_, 'mlt': mlt_}, cs='AACGM', **pcolormesh_config)
        # Add a color bar
        panel1.add_colorbar(ipc, c_label=band + " (R)", c_scale=pcolormesh_config['c_scale'], left=1.1, bottom=0.1,
                            width=0.05, height=0.7)

        # Overlay the gridlines
        panel1.overlay_gridlines(lat_res=5, lon_label_separator=5)

        # Overlay the coastlines in the AACGM coordinate
        panel1.overlay_coastlines()

        # Overlay cross-track velocity along satellite trajectory
        sc_dt = ds_s1['SC_DATETIME'].value.flatten()
        sc_lat = ds_s1['SC_GEO_LAT'].value.flatten()
        sc_lon = ds_s1['SC_GEO_LON'].value.flatten()
        sc_alt = ds_s1['SC_GEO_ALT'].value.flatten()
        sc_coords = {'lat': sc_lat, 'lon': sc_lon, 'height': sc_alt}

        v_H = ds_s1['v_i_H'].value.flatten()
        # panel1.overlay_cross_track_vector(
        #    vector=v_H, unit_vector=1000, vector_unit='m/s', alpha=0.3, color='red',
        #    sc_coords=sc_coords, sc_ut=sc_dt, cs='GEO',
        # )
        # Overlay the satellite trajectory with ticks
        orbit = OrbitPosition_SSCWS(dt_fr=dt_c - datetime.timedelta(minutes=20),
                                    dt_to=dt_c + datetime.timedelta(minutes=20), sat_id='dmsp' + sat_id.lower())
        sc_dt = orbit['SC_DATETIME'].value.flatten()
        sc_lat = orbit['SC_GEO_LAT'].value.flatten()
        sc_lon = orbit['SC_GEO_LON'].value.flatten()
        sc_alt = orbit['SC_GEO_ALT'].value.flatten()
        sc_coords = {'lat': sc_lat, 'lon': sc_lon, 'height': sc_alt}
        panel1.overlay_sc_trajectory(sc_ut=sc_dt, sc_coords=sc_coords, cs='GEO', time_minor_tick=None)

        # Overlay sites
        panel1.overlay_sites(site_ids=['TRO', 'ESR'],
                             coords={'lat': [69.58, 78.15], 'lon': [19.23, 16.02], 'height': [0, 0]},
                            cs='GEO', marker='^', markersize=3)

        # Add the title and save the figure
        polestr = 'North' if pole == 'N' else 'South'
        panel1.add_title(
            title='DMSP/SSUSI, ' + band + ', ' + sat_id.upper() + ', ' + polestr + ', ' + dt_c.strftime(
                '%Y-%m-%d %H%M UT'))

    except:
        print("Error in SSUSI data! Check the data availability first!")

    # Add TSDashboard
    try:
        diff_minutes = 20
        dt_fr_2 = dt_c - datetime.timedelta(minutes=diff_minutes / 2)
        dt_to_2 = dt_c + datetime.timedelta(minutes=diff_minutes / 2)
        dashboard_2 = TSDashboard(dt_fr=dt_fr_2, dt_to=dt_to_2, figure=dashboard.figure,
                                  timeline_extra_labels=['GEO_LAT', 'GEO_LON', 'AACGM_LAT', 'AACGM_MLT'])

        dataset_s1 = dashboard_2.dock(datasource_contents=['madrigal', 'satellites', 'dmsp', 's1'], sat_id=sat_id)
        # dataset_s4 = dashboard_2.dock(datasource_contents=['madrigal', 'dmsp', 's4'], sat_id=sat_id)
        dataset_e = dashboard_2.dock(datasource_contents=['madrigal', 'satellites', 'dmsp', 'e'], sat_id=sat_id)

        n_e = dashboard_2.assign_variable('n_e', dataset=dataset_s1)
        v_i_H = dashboard_2.assign_variable('v_i_H', dataset=dataset_s1)
        v_i_V = dashboard_2.assign_variable('v_i_V', dataset=dataset_s1)
        d_B_D = dashboard_2.assign_variable('d_B_D', dataset=dataset_s1)
        d_B_P = dashboard_2.assign_variable('d_B_P', dataset=dataset_s1)
        d_B_F = dashboard_2.assign_variable('d_B_F', dataset=dataset_s1)

        JE_e = dashboard_2.assign_variable('JE_e', dataset=dataset_e)
        JE_i = dashboard_2.assign_variable('JE_i', dataset=dataset_e)
        jE_e = dashboard_2.assign_variable('jE_e', dataset=dataset_e)
        jE_i = dashboard_2.assign_variable('jE_i', dataset=dataset_e)
        E_e_MEAN = dashboard_2.assign_variable('E_e_MEAN', dataset=dataset_e)
        E_i_MEAN = dashboard_2.assign_variable('E_i_MEAN', dataset=dataset_e)

        # T_i = dashboard_2.assign_variable('T_i', dataset=dataset_s4)
        # T_e = dashboard_2.assign_variable('T_e', dataset=dataset_s4)
        # c_O_p = dashboard_2.assign_variable('COMP_O_p', dataset=dataset_s4)

        layout = [
            [v_i_H, v_i_V],
            [d_B_P, d_B_D, d_B_F],
            [E_e_MEAN, E_i_MEAN],
            [JE_e, JE_i],
            [jE_e],
            [jE_i],
        ]
        dashboard_2.set_layout(panel_layouts=layout, left=0.5, right=0.9, hspace=0.01)
        dashboard_2.draw()
        uts = dashboard_2.search_UTs(AACGM_LAT=66.6, GEO_LON=[8, 32])
        if list(uts):
            dashboard_2.add_vertical_line(uts[0])
        uts = dashboard_2.search_UTs(AACGM_LAT=75.4, GEO_LON=[0, 30])
        if list(uts):
            dashboard_2.add_vertical_line(uts[0])
        dashboard_2.add_panel_labels()
    except:
        print("Error in SSJ and SSM data!")

    file_name = 'DMSP_' + dt_c.strftime(
        '%Y-%m-%d_%H%M') + '_' + sat_id.upper() + '_SSUSI_' + band + '_SSJ_SSM_SSIES_' + pole
    file_path_out = file_dir / file_name
    plt.savefig(file_path_out, dpi=300)
    print(f'file out:{file_path_out}')
    # show the figure
    # plt.show()


def search_record():
    root_dir = pathlib.Path("/home/lei/01-Work/01-Project/OY21-VisitingPhD/events/good_cases")
    root_dir = pathlib.Path("/home/lei/01-Work/01-Project/OY21-VisitingPhD/events/good_cases_waitlist")
    file_paths = root_dir.glob("**/*conjugation*.png")
    sat_ids = []
    dts = []
    file_dirs = []
    for ind, fp in enumerate(file_paths):
        fn = fp.stem
        print(fp)
        rc = re.compile("(F[\d]+)[\w\s]+([\d]{8}\s[\d]{6})")
        result = rc.findall(fn)
        if not list(result):
            continue
        sat_id = result[0][0]
        dt = datetime.datetime.strptime(result[0][1], "%Y%m%d %H%M%S")
        file_dirs.append(fp.parent.resolve())
        sat_ids.append(sat_id.lower())
        dts.append(dt)

    return file_dirs, sat_ids, dts


def good_events():
    file_dirs, sat_ids, dts = search_record()

    falsed_events = []
    for i, (fd, sat_id, dt) in enumerate(zip(file_dirs, sat_ids, dts)):
        try:
            show_dmsp(sat_id=sat_id, dt_c=dt, file_dir=fd)
        except:
            falsed_events.append([dt, fd, sat_id])
        # show_dmsp(sat_id=sat_id, dt_c=dt, file_dir=fd)

    for info in falsed_events:
        print(info)


if __name__ == "__main__":
    # good_events()
    sat_id = 'f18'
    orbit_id = '61116'
    dt = datetime.datetime.strptime('20210824 110700', "%Y%m%d %H%M%S")
    file_dir = pathlib.Path(__file__).parent.resolve()
    show_dmsp(sat_id=sat_id, dt_c=dt, file_dir=file_dir, orbit_id=orbit_id)


import datetime
import matplotlib.pyplot as plt

import geospacelab.visualization.mpl.dashboards as dashboards


def test():
    dt_fr = datetime.datetime(2016, 3, 14, 0)
    dt_to = datetime.datetime(2016, 3, 15, 23,)
    dashboard = dashboards.TSDashboard(dt_fr=dt_fr, dt_to=dt_to)
    ds1 = dashboard.dock(
        datasource_contents=['madrigal', 'isr', 'millstonehill', 'vi'])

    ds2 = dashboard.dock(
        datasource_contents=['madrigal', 'isr', 'millstonehill', 'gridded'])
    v_i_N = dashboard.assign_variable('v_i_N', dataset=ds1)
    v_i_E = dashboard.assign_variable('v_i_E', dataset=ds1)
    v_i_Z = dashboard.assign_variable('v_i_Z', dataset=ds1)
    E_E = dashboard.assign_variable('E_E', dataset=ds1)
    E_N = dashboard.assign_variable('E_N', dataset=ds1)

    n_e = dashboard.assign_variable('n_e', dataset=ds2)
    T_e = dashboard.assign_variable('T_e', dataset=ds2)
    T_i = dashboard.assign_variable('T_i', dataset=ds2)
    v_i_up = dashboard.assign_variable('v_i_UP', dataset=ds2)

    # ds2 = dashboard.dock(
    #     datasource_contents=['madrigal', 'isr', 'millstonehill', 'gridded'])
    layout = [[n_e], [v_i_up], [T_e], [T_i]]
    # layout = [[Bz, By], [v_sw], [n_p], [sym_h]]
    dashboard.set_layout(panel_layouts=layout, hspace=0.1)
    dashboard.draw()
    # dashboard.save_figure(file_name='example_omni_6', append_time=False)
    pass


def example(dt_fr, dt_to):

    #dt_fr = datetime.datetime.strptime('20160315' + '1200', '%Y%m%d%H%M')
    #dt_to = datetime.datetime.strptime('20160316' + '1200', '%Y%m%d%H%M')

    antenna = 'misa'
    pulse_code = 'alternating'
    pulse_length = 480
    load_mode = 'AUTO'
    dashboard = MillstoneHillISRDashboard(
        dt_fr, dt_to, antenna=antenna, pulse_code=pulse_code, pulse_length=pulse_length,
    )
    dashboard.select_beams(az_el_pairs=[[-40.5, 45]])
    dashboard.quicklook()
    dashboard.save_figure()
    # dashboard.show()

def example2():

    for i in range(6):
        dt0 = datetime.datetime(2016, 3, 13)
        example(dt_fr=dt0 + datetime.timedelta(days=i, hours=12), dt_to = dt0 + datetime.timedelta(days=i+1, hours=12))

if __name__ == '__main__':
    example2()



# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"


import copy
import geospacelab.toolbox.utilities.pylogging as mylog
import geospacelab.toolbox.utilities.pybasic as mybasic


def set_object_attributes(obj, *args, dict_deepcopy=True, **kwargs):
    append = kwargs.pop('append', False)
    logging = kwargs.pop('logging', True)

    append_rec = 0
    for ind, arg in enumerate(args):
        if ind % 2 == 0:
            kwargs[arg] = args[ind+1]
        else:
            continue
    for key, value in kwargs.items():
        if not hasattr(obj, key):
            if not append:
                mylog.StreamLogger.warning("Object %s: '%s' is not found in the named attributes!",
                                           mybasic.retrieve_name(obj), key)
                append_rec = 1
                continue
            else:
                if logging:
                    mylog.simpleinfo.info("Object %s: The attribute '%s' is added!", mybasic.retrieve_name(obj), key)

        if type(value) is dict and dict_deepcopy is True:
            value = copy.deepcopy(value)
        setattr(obj, key, value)

    if append_rec:
        mylog.simpleinfo.info("Object %s: To add the new attribute, set append=True", mybasic.retrieve_name(obj))


def get_object_attributes(obj):
    attrs = {}
    for name in vars(obj):
        if name.startswith("__"):
            continue
        if name.startswith("_"):
            continue
        attr = getattr(obj, name)
        if callable(attr):
            continue
        attrs[name] = attr
    return attrs


class StrBase(str):
    """
    Base class inherits from *str*. Useful when adding additional attributes to a string.
    """
    def __new__(cls, str_in):
        if issubclass(str_in.__class__, StrBase):
            obj = str_in
        else:
            obj = str.__new__(cls, str_in)
        return obj

    def config(self, logging=True, **kwargs):
        """
        A standard method used in this package to set attributes to the string.
        :param logging: If ''True'', show logging.
        :type logging: bool, default: True
        :param kwargs: Other optional keyword arguments in :meth:'~geospacelab.toolbox.utilities.pyclass.set_object_attributes'
        """
        set_object_attributes(self, append=False, logging=logging, **kwargs)



if __name__ == "__main__":
    class A(object):
        def __init__(self):
            self.a = 1
            
    a = A()
    b = A()
    set_object_attributes(a, 'b', 2, append=True)
    print(a.b)
    print(b.b)



# Licensed under the BSD 3-Clause License
# Copyright (C) 2021 GeospaceLab (geospacelab)
# Author: Lei Cai, Space Physics and Astronomy, University of Oulu

__author__ = "Lei Cai"
__copyright__ = "Copyright 2021, GeospaceLab"
__license__ = "BSD-3-Clause License"
__email__ = "lei.cai@oulu.fi"
__docformat__ = "reStructureText"


import datetime

from geospacelab.visualization.mpl.dashboards import TSDashboard


class OMNIDashboard(TSDashboard):
    def __init__(self, dt_fr, dt_to, **kwargs):
        figure = kwargs.pop('figure', 'new')
        figure_config = kwargs.pop('figure_config', {'figsize': (10, 8)})
        super().__init__(dt_fr=dt_fr, dt_to=dt_to, figure=figure, figure_config=figure_config)
        omni_config = {
            'omni_type': kwargs.pop('omni_type', 'OMNI2'),
            'omni_res': kwargs.pop('omni_res', '1min'),
            'load_mode': kwargs.get('load_mode', 'AUTO'),
            'allow_load': kwargs.get('allow_load', True)
        }
        ds_1 = self.dock(datasource_contents=['cdaweb', 'omni'], **omni_config)
        ds_2 = self.dock(datasource_contents=['wdc', 'ae'], allow_load=True, load_mode='AUTO')
        ds_3 = self.dock(datasource_contents=['wdc', 'asysym'])
        ds_4 = self.dock(datasource_contents=['gfz', 'kpap'])
        # ds_1.list_all_variables()
        self.title = kwargs.pop('title', ', '.join([ds_1.facility, ds_1.omni_res]))

    def list_all_variables(self):
        for key, dataset in self.datasets.items():
            dataset.list_all_variables()

    def save_figure(self, file_name=None, file_dir=None, append_time=True, **kwargs):
        if file_name is None:
            file_name = kwargs.pop('file_name', self.title.replace(', ', '_'))
        super().save_figure(file_name=file_name, file_dir=file_dir, append_time=append_time, **kwargs)

    def add_title(self, x=0.5, y=1.06, title=None, append_time=True, **kwargs):
        if title is None:
            title = self.title
        super().add_title(x=x, y=y, title=title, append_time=append_time, **kwargs)

    def quicklook(self):
        Bx = self.assign_variable('B_x_GSM', dataset=self.datasets[1])
        By = self.assign_variable('B_y_GSM', dataset=self.datasets[1])
        Bz = self.assign_variable('B_z_GSM', dataset=self.datasets[1])

        n_p = self.assign_variable('n_p', dataset=self.datasets[1])
        v_sw = self.assign_variable('v_sw', dataset=self.datasets[1])
        p_dyn = self.assign_variable('p_dyn', dataset=self.datasets[1])

        au = self.assign_variable('AU', dataset=self.datasets[2])
        au.visual.axis[1].lim = [None, None]
        al = self.assign_variable('AL', dataset=self.datasets[2])

        sym_h = self.assign_variable('SYM_H', dataset=self.datasets[3])
        sym_h.visual.axis[1].lim = [None, None]
        sym_h.visual.axis[1].label = '@v.label'

        kp = self.assign_variable('Kp', dataset=self.datasets[4])
        self.list_assigned_variables()
        self.list_datasets()

        layout = [[Bx, By, Bz], [v_sw], [n_p], [p_dyn], [au, al], [sym_h], [kp]]
        self.set_layout(panel_layouts=layout)
        # plt.style.use('dark_background')
        # dt_fr_1 = datetime.datetime.strptime('20201209' + '1300', '%Y%m%d%H%M')
        # dt_to_1 = datetime.datetime.strptime('20201210' + '1200', '%Y%m%d%H%M')

        self.draw()
        self.add_title()
        self.add_panel_labels()


def example():

    dt_fr = datetime.datetime.strptime('20180211' + '0600', '%Y%m%d%H%M')
    dt_to = datetime.datetime.strptime('20180212' + '0600', '%Y%m%d%H%M')

    omni_type = 'OMNI2'
    omni_res = '1min'
    load_mode = 'AUTO'
    dashboard = OMNIDashboard(
        dt_fr, dt_to, omni_type=omni_type, omni_res=omni_res, load_mode=load_mode
    )
    dashboard.quicklook()

    # save figure
    dashboard.save_figure()
    # show on screen
    dashboard.show()


if __name__ == '__main__':
    example()



import datetime
import matplotlib.pyplot as plt
import numpy as np

import geospacelab.visualization.mpl.geomap.geodashboards as geomap


def test_ampere():
    dt_fr = datetime.datetime(2016, 10, 23, 16)
    dt_to = datetime.datetime(2016, 10, 23, 23, 59)
    time1 = datetime.datetime(2016, 10, 23, 18, 10)
    pole = 'N'
    load_mode = 'assigned'
    # specify the file full path
    data_file_paths = ['/home/lei/afys-data/JHUAPL/AMPERE/Fitted/201610/20161023.0000.86400.120.north.grd.ncdf']

    viewer = geomap.GeoDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (8, 8)})
    # viewer.dock(datasource_contents=['jhuapl', 'dmsp', 'ssusi', 'edraur'], pole='N', sat_id='f17', orbit_id='46863')
    viewer.dock(datasource_contents=['jhuapl', 'ampere', 'fitted'], load_mode=load_mode, data_file_paths=data_file_paths)
    viewer.set_layout(1, 1)
    dataset_ampere = viewer.datasets[1]

    fac = viewer.assign_variable('GRID_Jr', dataset_index=1)
    dts = viewer.assign_variable('DATETIME', dataset_index=1).value.flatten()
    mlat = viewer.assign_variable('GRID_MLAT', dataset_index=1).value
    mlt = viewer.assign_variable(('GRID_MLT'), dataset_index=1).value

    ind_t = dataset_ampere.get_time_ind(ut=time1)
    # initialize the polar map
    panel1 = viewer.add_polar_map(row_ind=0, col_ind=0, style='mlt-fixed', cs='AACGM', mlt_c=0., pole=pole, ut=time1, boundary_lat=60, mirror_south=True)

    panel1.overlay_coastlines()

    fac_ = fac.value[ind_t, :, :]
    # re-grid the original data with higher spatial resolution, default mlt_res = 0.05, mlat_res = 0.5. used for plotting.
    grid_mlat, grid_mlt, grid_fac = dataset_ampere.grid_fac(fac_, mlt_res=0.05, mlat_res=0.05, interp_method='linear')

    # remove values less than 0.2
    grid_fac[np.abs(grid_fac)<0.2] = np.nan
    # fac_[np.abs(fac_) < 0.2] = np.nan
    pcolormesh_config = fac.visual.plot_config.pcolormesh
    pcolormesh_config.update(c_scale='linear')
    pcolormesh_config.update(c_lim=[-1, 1])
    pcolormesh_config.update(shading='auto')
    import geospacelab.visualization.mpl.colormaps as cm
    cmap = cm.cmap_gist_ncar_modified()
    cmap = 'jet'
    pcolormesh_config.update(cmap=cmap)
    # ipc = panel1.add_pcolor(fac_, coords={'lat': mlat[ind_t, ::], 'lon': None, 'mlt': mlt[ind_t, ::], 'height': 250.}, cs='AACGM', **pcolormesh_config)
    ipm = panel1.overlay_pcolormesh(grid_fac, coords={'lat': grid_mlat, 'lon': None, 'mlt': grid_mlt, 'height': 250.}, cs='AACGM', **pcolormesh_config)
    panel1.add_colorbar(ipm, c_label=r'FAC ($\mu$A/m$^2$)', c_scale=pcolormesh_config['c_scale'], left=1.1, bottom=0.1,
                        width=0.05, height=0.7)

    panel1.overlay_gridlines(lat_res=5, lon_label_separator=5)

    polestr = 'North' if pole == 'N' else 'South'
    plt.savefig('ampere_example', dpi=300)
    plt.show()


if __name__ == "__main__":
    test_ampere()



import datetime
import matplotlib.pyplot as plt
import numpy as np

import geospacelab.visualization.mpl.dashboards as dashboards


def test_swarm():
    dt_fr = datetime.datetime(2014, 8, 28, 7, 16)
    dt_to = datetime.datetime(2014, 8, 28, 7, 36)
    # specify the file full path

    db = dashboards.TSDashboard(dt_fr=dt_fr, dt_to=dt_to, figure_config={'figsize': (8, 8)})

    db.dock(datasource_contents=['esa_eo', 'swarm', 'advanced', 'efi_lp_hm'], product='LP_HM', sat_id='A', quality_control=False)

    n_e = db.assign_variable('n_e')
    T_e = db.assign_variable('T_e')
    flag = db.assign_variable('QUALITY_FLAG')
    
    ds_tii = db.dock(
        datasource_contents=['esa_eo', 'swarm', 'advanced', 'efi_tct02'], 
        product='TCT02', sat_id='A', quality_control=False
        )

    v_i_H_x = db.assign_variable('v_i_H_x', dataset=ds_tii)
    v_i_H_y = db.assign_variable('v_i_H_y', dataset=ds_tii)
    v_i_V_x = db.assign_variable('v_i_V_x', dataset=ds_tii)
    v_i_V_z = db.assign_variable('v_i_V_z', dataset=ds_tii)

    db.set_layout(panel_layouts=[[n_e], [T_e], [v_i_H_x, v_i_H_y, v_i_V_x, v_i_V_z], [flag]])
    db.draw()
    plt.savefig('swarm_example', dpi=300)
    plt.show()


if __name__ == "__main__":
    test_swarm()
